import { ByteLength } from './common'

export class BufferReader {
  private readonly _data: Buffer
  private _cursor: number

  constructor (data: Buffer = Buffer.from([])) {
    this._data = data
    this._cursor = 0
  }

  get cursor (): number {
    return this._cursor
  }

  set cursor (position: number) {
    if (position < 0 || position > this._data.length) {
      throw RangeError(`Cannot seek to ${this.cursor} of ${this._data.length} bytes.`)
    }
    this._cursor = position
  }

  readUInt16 (): number {
    this.cursor += ByteLength.UInt16
    return this._data.slice(this.cursor - ByteLength.UInt16, this.cursor).readInt16BE()
  }

  readUInt32 (): number {
    this.cursor += ByteLength.UInt32
    return this._data.slice(this.cursor - ByteLength.UInt32, this.cursor).readInt32BE()
  }

  readBuffer (len: number): Buffer {
    this.cursor += len
    return this._data.slice(this.cursor - len, this.cursor)
  }
}
